import 'dart:ffi' as ffi;

import 'bindings.dart' as ffi;
import 'gio.dart';
import 'variant.dart';
import 'variant_type.dart';

class GVariantBuilder {
  final ffi.Pointer<ffi.GVariantBuilder> _ptr;

  GVariantBuilder.fromType(GVariantType type)
      : _ptr = gio.g_variant_builder_new(type.toPointer());

  GVariantBuilder.fromPointer(this._ptr);
  ffi.Pointer<ffi.GVariantBuilder> toPointer() => _ptr;

  static GVariant build(GVariantType type, GVariant value) {
    return buildAll(type, <GVariant>[value]);
  }

  static GVariant buildAll(GVariantType type, Iterable<GVariant> values) {
    final builder = GVariantBuilder.fromType(type);
    builder.addValues(values);
    final res = builder.end();
    builder.dispose();
    return res;
  }

  void addValue(GVariant value) {
    gio.g_variant_builder_add_value(_ptr, value.toPointer());
  }

  void addValues(Iterable<GVariant> values) {
    for (final value in values) {
      addValue(value);
    }
  }

  GVariant end() {
    return GVariant.fromPointer(gio.g_variant_builder_end(_ptr));
  }

  void dispose() => gio.g_variant_builder_unref(_ptr);
}
