import 'dart:ffi' as ffi;
import 'dart:io';

import 'package:gsettings/gsettings.dart';
import 'package:more/more.dart';
import 'package:test/test.dart';

import 'test_utils.dart';

void main() {
  setUp(() => setUpGSettings());

  test('instance', () {
    final schema1 = GSettingsSchema.lookup('gsettings.dart.empty')!;
    final pointer1 = schema1.toPointer();
    expect(pointer1, isNot(equals(ffi.nullptr)));

    final schema2 = GSettingsSchema.fromPointer(pointer1);
    final pointer2 = schema2.toPointer();
    expect(pointer2, equals(pointer1));

    schema1.dispose();
    // ### TODO: schema2.dispose();
  });

  test('lookup', () {
    expect(GSettingsSchema.lookup('foo.bar'), isNull);
  });

  test('id & path', () {
    final types = GSettingsSchema.lookup('gsettings.dart.types')!;
    expect(types.id, equals('gsettings.dart.types'));
    expect(types.path, equals('/gsettings/dart/types/'));
    expect(types, equals(types));

    final same = GSettingsSchema.lookup('gsettings.dart.types')!;
    expect(same.id, equals('gsettings.dart.types'));
    expect(same.path, equals('/gsettings/dart/types/'));
    expect(same, equals(types));

    final none = GSettingsSchema.lookup('gsettings.dart.none')!;
    expect(none.id, equals('gsettings.dart.none'));
    expect(none.path, isNull);
    expect(none, isNot(equals(same)));

    final empty = GSettingsSchema.lookup('gsettings.dart.empty')!;
    expect(empty.id, equals('gsettings.dart.empty'));
    expect(empty.path, equals('/gsettings/dart/empty/'));
    expect(empty, isNot(equals(same)));
    expect(empty, isNot(equals(none)));

    types.dispose();
    same.dispose();
    none.dispose();
    empty.dispose();
  });

  test('keys', () {
    final schema = GSettingsSchema.lookup('gsettings.dart.types')!;
    expect(schema.hasKey('bool-value'), isTrue);
    expect(schema.hasKey('non-existent-value'), isFalse);
    expect(
        schema.keys,
        unorderedEquals(<String>[
          'array-int-values',
          'tuple7-values',
          'bool-value',
          'nullable-int',
          'double-value',
          'array-string-values',
          'tuple2-values',
          'int32-value',
          'tuple5-values',
          'tuple8-values',
          'byte-value',
          'uint16-value',
          'tuple3-values',
          'tuple9-values',
          'dict-string-int-values',
          'tuple6-values',
          'dict-int-int-values',
          'int16-value',
          'string-value',
          'nullable-string',
          'int64-value',
          'dict-string-string-values',
          'tuple4-values',
          'uint32-value',
          'int-with-range',
        ]));
    schema.dispose();
  });

  test('children', () {
    final schema = GSettingsSchema.lookup('gsettings.dart.parent')!;
    expect(schema.children, <String>['child1', 'child2']);
    schema.dispose();
  });

  test('key', () {
    final schema = GSettingsSchema.lookup('gsettings.dart.types')!;

    final boolean = schema.key('bool-value');
    expect(boolean.toPointer(), isNot(equals(ffi.nullptr)));
    expect(boolean.name, equals('bool-value'));
    expect(boolean.summary, equals('A boolean value'));
    expect(boolean.description, equals('This is a boolean value for testing.'));
    expect(boolean.defaultValue, isTrue);

    final nullable = schema.key('nullable-int');
    expect(nullable.defaultValue, isNull);

    boolean.dispose();
    nullable.dispose();
    schema.dispose();
  });

  test('range', () {
    final schema = GSettingsSchema.lookup('gsettings.dart.types')!;

    final key = schema.key('int-with-range');
    expect(key.minimumValue, equals(1));
    expect(key.maximumValue, equals(10));

    expect(key.rangeCheck(0), isFalse);
    expect(key.rangeCheck(1), isTrue);
    expect(key.rangeCheck(5), isTrue);
    expect(key.rangeCheck(10), isTrue);
    expect(key.rangeCheck(11), isFalse);

    key.dispose();
    schema.dispose();
  });
}
